@@ -1,6 +1,5 @@
 package Text::Xslate::Parser;
 use Any::Moose;
-use Any::Moose '::Util::TypeConstraints';
 
 use Scalar::Util ();
 use Text::Xslate::Symbol;
@@ -56,11 +55,6 @@ my $COMMENT = qr/\# [^\n;]* (?=[;\n])?/xms;
 
 my $CODE    = qr/ (?: (?: $STRING | [^'"] )*? ) /xms; # ' for poor editors
 
-my $RegexpRefType = subtype __PACKAGE__ . '.RegexpRef' => as 'Maybe[RegexpRef]';
-coerce $RegexpRefType =>
-    from 'Str', via { qr/\Q$_\E/xms },
-;
-
 has [qw(compiler engine)] => (
     is       => 'rw',
     required => 0,
@@ -128,27 +122,27 @@ has input => (
 
 has line_start => (
     is      => 'ro',
-    isa     => $RegexpRefType,
+    isa     => 'Maybe[Str]',
     coerce  => 1,
     builder => '_build_line_start',
 );
-sub _build_line_start { qr/\Q:/xms }
+sub _build_line_start { ':' }
 
 has tag_start => (
     is      => 'ro',
-    isa     => $RegexpRefType,
+    isa     => 'Str',
     coerce  => 1,
     builder => '_build_tag_start',
 );
-sub _build_tag_start { qr/\Q<:/xms }
+sub _build_tag_start { '<:' }
 
 has tag_end => (
     is      => 'ro',
-    isa     => $RegexpRefType,
+    isa     => 'Str',
     coerce  => 1,
     builder => '_build_tag_end',
 );
-sub _build_tag_end { qr/\Q:>/xms }
+sub _build_tag_end { ':>' }
 
 has shortcut_table => (
     is      => 'ro',
@@ -205,11 +199,11 @@ sub split :method {
     my $tag_start     = $parser->tag_start;
     my $tag_end       = $parser->tag_end;
 
-    my $lex_line_code = defined($line_start) && qr/\A ^ [ \t]* $line_start ([^\n]* \n?) /xms;
-    my $lex_tag_start = qr/\A $tag_start ($CHOMP_FLAGS?)/xms;
-    my $lex_tag_end   = qr/\A ($CODE) ($CHOMP_FLAGS?) $tag_end/xms;
+    my $lex_line_code = defined($line_start) && qr/\A ^ [ \t]* \Q$line_start\E ([^\n]* \n?) /xms;
+    my $lex_tag_start = qr/\A \Q$tag_start\E ($CHOMP_FLAGS?)/xms;
+    my $lex_tag_end   = qr/\A ($CODE) ($CHOMP_FLAGS?) \Q$tag_end\E/xms;
 
-    my $lex_text = qr/\A ( [^\n]*? (?: \n | (?= $tag_start ) | \z ) ) /xms;
+    my $lex_text = qr/\A ( [^\n]*? (?: \n | (?= \Q$tag_start\E ) | \z ) ) /xms;
 
     my $in_tag = 0;
 
@@ -220,7 +214,7 @@ sub split :method {
 
                 my($code, $chomp) = ($1, $2);
 
-                push @tokens, [ code => $parser->trim_code($code) ];
+                push @tokens, [ code => $code ];
                 if($chomp) {
                     push @tokens, [ postchomp => $chomp ];
                 }
@@ -232,8 +226,7 @@ sub split :method {
         }
         # not $in_tag
         elsif($lex_line_code && s/$lex_line_code//xms) {
-            push @tokens,
-                [ code => $parser->trim_code($1) ];
+            push @tokens, [ code => $1 ];
         }
         elsif(s/$lex_tag_start//xms) {
             $in_tag = 1;
@@ -244,7 +237,7 @@ sub split :method {
             }
         }
         elsif(s/$lex_text//xms) {
-            push @tokens, [ text => $1 ];
+            push @tokens, [ text => $1 ] if length($1);
         }
         else {
             confess "Oops: Unreached code, near" . p($_);
@@ -305,6 +298,8 @@ sub preprocess {
             $s =~ s/$shortcut_rx/$shortcut_table->{$1}/xms
                 if $shortcut;
 
+            $s = $parser->trim_code($s);
+
             if($s =~ /\A \s* [}] \s* \z/xms){
                 $code .= $s;
             }
@@ -349,7 +344,7 @@ sub parse {
     my $ast = $parser->statements();
 
     if($parser->input ne '') {
-        $parser->_error("Syntax error", $parser->token);
+        $parser->_error("Syntax error", $parser->token, $parser->input);
     }
 
     return $ast;
@@ -651,6 +646,27 @@ sub advance {
      ) );
 }
 
+sub default_nud {
+    my($parser, $symbol) = @_;
+    return $symbol; # as is
+}
+
+sub default_led {
+    my($parser, $symbol) = @_;
+    $parser->near_token($parser->token);
+    $parser->_error(
+        sprintf 'Missing operator (%s): %s',
+        $symbol->arity, $symbol->id);
+}
+
+sub default_std {
+    my($parser, $symbol) = @_;
+    $parser->near_token($parser->token);
+    $parser->_error(
+        sprintf 'Not a statement (%s): %s',
+        $symbol->arity, $symbol->id);
+}
+
 sub expression {
     my($parser, $rbp) = @_;
 
@@ -1703,7 +1719,6 @@ sub _error {
         $parser->file, $parser->line, $message, $near);
 }
 
-no Any::Moose '::Util::TypeConstraints';
 no Any::Moose;
 __PACKAGE__->meta->make_immutable;
 __END__
